package com.mobai.mq.rabbitmq.rabbitMq;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.mobai.domain.StayTime;
import com.mobai.service.StayTimeService;
import com.mobai.util.RedisService;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 消费者：消息处理器
 *
 * @ClassName MessageHandler
 * @Description 描述
 * @Author Mobai
 * @Date 2024/5/31 14:37
 */
@Log4j2
@Component
public class MessageHandler {

    @Autowired
    private StayTimeService stayTimeService;

    @Autowired
    private RedisService redisService;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    /**
     * 上线事件
     *
     * @param msg
     */
    @RabbitListener(queues = {"event"})
    private void trainUp(String msg, Message message, Channel channel) {
        log.info("event:{}", msg);
        // 事件内容
        JSONObject jsonObject = JSON.parseObject(msg);
        String clientId = JSON.to(String.class, jsonObject.get("clientId"));
        if (!(clientId.contains("-") && clientId.contains("."))) {
            log.error("不是车辆事件：：{}", message);
            try {
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                return;
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        String[] split = clientId.split("-");
        String vin = split[0];
        String ip = split[1];
        //判断业务是否存入redis存配置
        if (redisService.hasKey(vin+"List")||redisService.getList(vin+"List")==null){
            log.warn("该车辆无配置信息，已通知业务，车辆为：{}",vin);
            rabbitTemplate.convertAndSend("zhiLian-vehicle-vin",vin);
        }
        long timestamp = JSON.to(Long.class, jsonObject.get("timestamp"));
        // 判断登陆事件
        if (jsonObject.get("auth") != null) {
            try {
                log.info("上线事件");
                log.info("上线车辆vin:{}--上线时ip:{}", vin, ip);
                if (ip == null) {
                    throw new ServletException("上线时ip为空");
                }
                // 方法内有判断，有则自增，无则创建
                redisService.increment("onlineCar-" + ip, 1);
                boolean save = stayTimeService.save(new StayTime() {{
                    setIp(ip);
                    setVin(vin);
                    setUpTime(timestamp);
                }});
                // 发送事件开启指标预警事件
                if (save) {
                    rabbitTemplate.convertAndSend("standard-Warn-Event-Start", vin);
                }
                log.info(save ? vin + "上线记录成功" : vin + "上线记录失败");
                // 消息消费成功则确认
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            } catch (IOException | ServletException e) {
                log.error("上线失败");
                try {
                    channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, false);
                } catch (IOException ex) {
                    throw new RuntimeException(ex);
                }
                throw new RuntimeException(e);
            }
        } else {
            log.info("下线事件");
            try {
                log.info("下线车辆vin:{}--下线时ip:{}", vin, ip);
                if (ip == null) {
                    throw new ServletException("下线时ip为空");
                }
                // 方法内有判断，有则自减，无则创建
                redisService.decrement("onlineCar-" + ip, 1);
                // 判断是否有该记录
                StayTime vinStayTime = stayTimeService.getOne(new LambdaQueryWrapper<>() {{
                    eq(StayTime::getVin, vin);
                    eq(StayTime::getDownTime, 0);
                }});
                // 为空抛异常
                if (vinStayTime == null) {
                    throw new ServletException("{" + vin + "}数据不存在");
                }
                String format = new SimpleDateFormat("HH时mm分ss秒").format(new Date(timestamp - vinStayTime.getUpTime() - 8 * 60 * 60 * 1000));
                vinStayTime.setStayLongTime(format);
                vinStayTime.setDownTime(timestamp);
                // 修改下线时间
                boolean update = stayTimeService.update(vinStayTime, new LambdaUpdateWrapper<>() {{
                    eq(StayTime::getVin, vin);
                    eq(StayTime::getDownTime, 0);
                }});
                // 输出在线时长
                log.info("车辆在线时长为：{}", format);
                log.info(update ? vin + "下线记录成功" : vin + "下线记录失败");
                // 发送事件开启指标预警事件
                if (update) {
                    rabbitTemplate.convertAndSend("standard-Warn-Event-End", vin);
                }
                redisService.deleteObject(vin);
                channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
            } catch (ServletException | IOException e) {
                log.error("下线失败");
                throw new RuntimeException(e);
            } finally {
                try {
                    channel.basicReject(message.getMessageProperties().getDeliveryTag(), false);
                } catch (IOException ex) {
                    throw new RuntimeException(ex);
                }
            }
        }
    }
}
